"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const pretty_data_1 = require("pretty-data");
const idin_protocol_1 = require("./idin-protocol");
const constants_1 = require("./constants");
function formatTransactionProtocolXML(issuerID, transactionID, requestedService = constants_1.REQUESTED_SERVICE) {
    const xml = pretty_data_1.pd.xmlmin(`
    <?xml version="1.0" encoding="UTF-8"?>
    <AcquirerTrxReq version="1.0.0"
      productID="NL:BVN:BankID:1.0"
      xmlns="http://www.betaalvereniging.nl/iDx/messages/Merchant-Acquirer/1.0.0"
      xmlns:ds="http://www.w3.org/2000/09/xmldsig#">
      <createDateTimestamp>${new Date().toISOString()}</createDateTimestamp>
      <Issuer>
        <issuerID>${issuerID}</issuerID>
      </Issuer>
      <Merchant>
        <merchantID>${constants_1.MERCHANT_ID}</merchantID>
        <subID>0</subID>
        <merchantReturnURL>${constants_1.MERCHANT_RETURN_URL}</merchantReturnURL>
      </Merchant>
      <Transaction>
        <expirationPeriod>${constants_1.EXPIRATION_PERIOD}</expirationPeriod>
        <language>${constants_1.DEFAULT_LANGUAGE}</language>
        <entranceCode>${constants_1.ID_PREFIX}${transactionID}</entranceCode>
        <container>
          <samlp:AuthnRequest
            xmlns:samlp="urn:oasis:names:tc:SAML:2.0:protocol"
            xmlns:saml="urn:oasis:names:tc:SAML:2.0:assertion"
            AttributeConsumingServiceIndex="${requestedService}"
            ID="${constants_1.ID_PREFIX}${transactionID}"
            IssueInstant="${new Date().toISOString()}"
            Version="2.0"
            ProtocolBinding="nl:bvn:bankid:1.0:protocol:iDx"
            AssertionConsumerServiceURL="${constants_1.MERCHANT_RETURN_URL}">
            <saml:Issuer>${constants_1.MERCHANT_ID}</saml:Issuer>
            <samlp:RequestedAuthnContext Comparison="minimum">
              <saml:AuthnContextClassRef>nl:bvn:bankid:1.0:${constants_1.LOA}</saml:AuthnContextClassRef>
            </samlp:RequestedAuthnContext>
          </samlp:AuthnRequest>
        </container>
      </Transaction>
    </AcquirerTrxReq>
  `);
    return idin_protocol_1.signXML(xml);
}
exports.formatTransactionProtocolXML = formatTransactionProtocolXML;
//# sourceMappingURL=transaction-protocol.js.map